import {Injectable, signal, WritableSignal} from '@angular/core';
import {HttpClient, HttpParams} from "@angular/common/http";
import {tap} from "rxjs";
import {Task} from "../interface/task"
import {Response} from "../interface/response"

@Injectable({
  providedIn: 'root'
})
export class TaskService {

  readonly url_task_list = '/api/task/list'
  task_list: WritableSignal<{ total: number, list: Task[], page: number, size: number }> = signal({
    page: 0,
    size: 20,
    total: 0,
    list: [] as Task[],
  })

  constructor(
    private http: HttpClient,
  ) {
    this.get_task_list().subscribe()
  }

  get_task_list(page: number = 0, size: number = 20) {
    const params = new HttpParams()
    params.set("page", page)
    params.set("size", size)
    return this.http.get<Response<{ total: number, list: Task[] }>>(this.url_task_list, {params: params}).pipe(
      tap({
        next: (res => {
          if (res.status === 200) {
            this.task_list.set({...this.task_list(), list: res.data.list, total: res.data.total})
          }
        })
      }),
    )
  }
}
